#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json
from django import template
from bigbuild.models import Page
from django.utils.safestring import mark_safe
from django.utils.html import conditional_escape
register = template.Library()


@register.simple_tag
def omniture_section(slug):
    """
    Accepts a Los Angeles Times slug and returns the section name that
    should be submitted to our Omniture analytics system.
    """
    # Cross walk the start of slugs to omniture sections
    prefixes = {
        'la-me-': 'local',
        'la-na-pol-': 'politics',
        'la-na': 'nation',
        'la-pol-': 'politics',
        'la-ed-': 'opinion',
        'la-oe-': 'opinion',
        'la-fi-': 'business',
        'la-fg-': 'world',
        'la-ca-': 'entertainment',
        'la-et-': 'entertainment',
        'la-sp-': 'sports',
        'la-hm-': 'home',
        'la-fo-': 'food',
        'la-tr-': 'travel',
        'la-ig-': 'fashion',
        'la-sci-': 'science'
    }
    try:
        section = next(
            prefixes.get(i) for i in prefixes.keys() if slug.startswith(i)
        ) or 'local'
    except StopIteration:
        section = 'local'
    return section


@register.filter(needs_autoescape=True)
def dropcap(text, autoescape=True):
    """
    Wraps the first character in a <span> tag with a .dropcap class
    that can be used to make it bigger than the surrounding text.
    """
    if not text:
        return ''
    first, other = text[0], text[1:]
    if autoescape:
        esc = conditional_escape
    else:
        def esc(x): return x
    template = "<span class='dropcap'>{}</span>{}"
    html = template.format(esc(first.upper()), esc(other))
    return mark_safe(html)


@register.filter
def jsonify(obj):
    """
    Returns an object in JSON format.
    """
    if isinstance(obj, Page):
        return json.dumps(obj.to_json())
    return json.dumps(obj)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import shutil
from django.http import Http404
from django.conf import settings
from django.views.static import serve
from bigbuild.models import PageList, Page, RetiredPage
from bigbuild.management.commands.build import Command as Build
from bakery.views import (
    BuildableTemplateView,
    BuildableDetailView,
    Buildable404View,
    BuildableRedirectView
)


class Static404View(Buildable404View):
    """
    The 404 page.
    """
    template_name = "bigbuild/404.html"
    build_path = "404.html"


class RobotsView(BuildableTemplateView):
    """
    The robots.txt file.
    """
    template_name = "bigbuild/robots.txt"
    build_path = "projects/robots.txt"


class IndexRedirectView(BuildableRedirectView):
    """
    Redirects the root URL to /projects/
    """
    build_path = "index.html"
    url = "/projects/"


class PageListView(BuildableTemplateView):
    """
    Renders a homepage with a list of all the pages.
    """
    template_name = "bigbuild/page_list.html"
    build_path = "projects/index.html"

    def get_context_data(self):
        PAGE_PUBLICATION_STATUS = getattr(
            settings,
            'PAGE_PUBLICATION_STATUS',
            'working'
        )
        return {
            'object_list': PageList(),
            'PAGE_PUBLICATION_STATUS': PAGE_PUBLICATION_STATUS
        }


class PageDetailView(BuildableDetailView):
    """
    Renders one of the page objects as an HTML response.
    """
    def get_queryset(self):
        return PageList()

    def get_object(self):
        qs = self.get_queryset()
        return qs[self.kwargs.get("slug")]

    def get_template_names(self):
        return [os.path.join(self.object.slug, "index.html")]

    def get_context_data(self, object=None):
        return {
            'metadata': self.object.metadata,
            'object': self.object,
            'STATIC_URL': self.object.get_static_url()
        }

    def build_static_directory(self, obj):
        """
        Builds an object's static subdirectory.
        """
        source_dir = obj.static_path
        target_dir = os.path.join(
            settings.BUILD_DIR,
            'projects',
            obj.slug,
            'static'
        )
        if settings.BAKERY_GZIP:
            Build().copytree_and_gzip(
                source_dir,
                target_dir
            )
        else:
            os.path.exists(target_dir) and shutil.rmtree(target_dir)
            shutil.copytree(source_dir, target_dir)

    def build_object(self, obj):
        if isinstance(obj, Page):
            super(PageDetailView, self).build_object(obj)
            self.build_static_directory(obj)
        elif isinstance(obj, RetiredPage):
            target = os.path.join(settings.BUILD_DIR, 'projects', obj.slug)
            os.path.exists(target) and shutil.rmtree(target)
            if settings.BAKERY_GZIP:
                Build().copytree_and_gzip(
                    obj.directory_path,
                    target
                )
            else:
                shutil.copytree(obj.directory_path, target)

    def build_queryset(self):
        [self.build_object(o) for o in self.get_queryset()]


class PageRetireView(PageDetailView):

    def get_context_data(self, object=None):
        context = super(PageRetireView, self).get_context_data(object=object)
        context['RETIREMENT'] = True
        return context


def page_static_serve(request, slug, path):  # pragma: no cover
    """
    Serves files from the ./static/ subdirectory for every page.
    """
    # Set the path to the file
    path = os.path.join(slug, 'static', path)
    # First test if it can be found in the PAGE_DIR
    try:
        return serve(
            request,
            path,
            document_root=settings.PAGE_DIR,
            show_indexes=True
        )
    # If it can't try to fallback to the RETIRED_DIR
    except Http404:
        return serve(
            request,
            path,
            document_root=settings.RETIRED_DIR,
            show_indexes=True
        )


class AdIframeView(BuildableTemplateView):
    """
    Builds the weird custom ad iframe we need to keep Tribune in business.
    """
    template_name = "ngux-tophat/ads/ngux-tophat-ad-iframe.html"
    build_path = "projects/static/ngux-tophat-ad-iframe.html"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from bigbuild.models import BasePage


class RetiredPage(BasePage):
    """
    A retired custom page published via static.latimes.com
    """
    # The directory where pages are stored
    DIR_NAME = 'RETIRED_DIR'
